/*-----------------------------------------------------------------------*/
/* XDIR v1.10 - This Directory utility enhances the function of the DIR  */
/*              command.  I utilized TURBO C version 2.0 and I also used */
/*              the CXL v5.2 external library for some of the advanced   */
/*              features.  This version is still in BETA testing.        */
/*                                                                       */
/*              In the next version I will be adding multiple embedded   */
/*              command arguments along with sorting and printing options*/
/*              I would like to thank Mike Smedley for making CXL so easy*/
/*              to use.  My hopes for this program is a clone of the     */
/*              PC-TOOLS environment, with less overhead, and utilization*/
/*              of Expanded Memory.                                      */
/*              If you have any comments or suggestions for enhancements */
/*              or improvements for later versions please drop me a line.*/
/*                                                                       */
/*              Copyright (c) 1990   Michael A. Couture                  */
/*                                   609 Ocean Ave.                      */
/*                                   Portland, Maine 04103               */
/*         												   */
/*       I can be reached at Northern Lights BBS - Peaks Island, Me      */
/*-----------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <dos.h>
#include <dir.h>
#include <process.h>
#include <conio.h>
#include "cxldef.h"
#include "cxlkey.h"
#include "cxlmou.h"
#include "cxlstr.h"
#include "cxlvid.h"
#include "cxlwin.h"
/*-----------------------------------------------------------------------*/
/*------------------------- FUNCTION PROTOTYPES -------------------------*/
/*-----------------------------------------------------------------------*/

void main(int argc, char *argv[]);
void parse_options(int argc, char *argv[]);
int  parse_filenames(struct ffblk *file_list);
int  pdate(struct ffblk *file_list);
int  ptime(struct ffblk *file_list);
int  add_sizes(struct ffblk *file_list);
int  error_exit(void);
void find_files(char[], char);
void print_directory(void);
void print_attribs(unsigned);
void get_disk_space(void);

static struct dfree dfinfo;                      /* Disk Space Structure */
static struct ffblk file_list[1024];             /* File List Structure  */
static char filename[128] = "*.*";               /* File Name            */
static char set_attrib = FA_DIREC;               /* File Attributes      */
static int count = 0;                            /* File Count           */
static long int total_size = 0;                  /* Total File Sizes     */

/*-----------------------------------------------------------------------*/
/*MAIN() - All you 'C' programmers know what this function does!         */
/*-----------------------------------------------------------------------*/
void main(int argc, char *argv[])
{
	parse_options(argc, argv);
}
/*-----------------------------------------------------------------------*/
/*PARSE_OPTIONS() - This function searches for the command-line arguments*/
/*                  and selects the correct option(s).                   */
/*-----------------------------------------------------------------------*/
void parse_options(int argc, char *argv[])
{
	char *arg;
		while(*argv[1] == '/') {
			for(arg = argv[1]; *arg; arg++) {
				switch(tolower(*arg)) {
					case 'd' : set_attrib=(FA_DIREC);
							 break;
					case 'h' : set_attrib=(1|2);
							 break;
					case 'r' : set_attrib=(1|2|3);
							 break;
					case 's' : set_attrib=(7);
							 break;
					case 'f' : get_disk_space();
							 break;
					case '?' : error_exit();
							 exit(0);
					default  : break;
				}
			}
			argc--;
			argv++;
		}
	if(argc > 1)
		if(strlen(argv[1]) < 128) {
			strcpy(filename, argv[1]);
		}
	find_files(filename, set_attrib);
}
/*-----------------------------------------------------------------------*/
/*FIND_FILES() - This function actually retrieves the files and stores   */
/*               them in the array, FILEINFO.                            */
/*-----------------------------------------------------------------------*/
void find_files(char filename[], char attribs)
{
	static int i;
	auto struct ffblk fileinfo;       /*    setup structure fileinfo */
	int done;

	done = findfirst(filename, &fileinfo, attribs);

	while(!done)
	{
		strcpy(file_list[i].ff_name, fileinfo.ff_name);
		file_list[i].ff_attrib = fileinfo.ff_attrib;
		file_list[i].ff_ftime  = fileinfo.ff_ftime;
		file_list[i].ff_fdate  = fileinfo.ff_fdate;
		file_list[i].ff_fsize  = fileinfo.ff_fsize;
		++i;
		done = findnext(&fileinfo);
	}
	print_directory();
}
/*-----------------------------------------------------------------------*/
/*PRINT_DIRECTORY() - This function controls the printing of the files   */
/*-----------------------------------------------------------------------*/
void print_directory(void)
{
	auto int x;
	x = 0;

	while(*file_list[x].ff_name != '\0')
	 {
		if(file_list[x].ff_attrib == FA_DIREC)
		{
			parse_filenames(&file_list[x]);
			printf("  <DIRECTORY> ");
			ptime(&file_list[x]);
			pdate(&file_list[x]);
			print_attribs(file_list[x].ff_attrib);
		 }
		 else if(file_list[x].ff_attrib == FA_LABEL)
		 {
			parse_filenames(&file_list[x]);
			printf(" <VOLUME LABEL>");
			ptime(&file_list[x]);
			pdate(&file_list[x]);
			print_attribs(file_list[x].ff_attrib);
		 }
		else
		{
			parse_filenames(&file_list[x]);
			printf("  %12u ", file_list[x].ff_fsize);
			ptime(&file_list[x]);
			pdate(&file_list[x]);
			print_attribs(file_list[x].ff_attrib);
			}
	add_sizes(&file_list[x]);
	++x;
	}
	printf("\nTotal files %d using %lu bytes\n",count, total_size);
}
/*-----------------------------------------------------------------------*/
/*PTIME() - This function does bit shifting for the files' time          */
/*-----------------------------------------------------------------------*/
int ptime(struct ffblk *file_list)
{
	static unsigned int min, sec, hour;

	hour = (((*file_list).ff_ftime >> 11) & 0x1f);
	min  = (((*file_list).ff_ftime >>  5) & 0x3f);
	sec  =  ((*file_list).ff_ftime & 0x1f);
	printf(" %02u:%02u:%02u ", hour, min, sec);
	return(0);
}
/*-----------------------------------------------------------------------*/
/*PDATE() - This function does bit shifting for the files' date          */
/*-----------------------------------------------------------------------*/
int pdate(struct ffblk *file_list)
{
	static unsigned int year, month, day;

	year  = (((*file_list).ff_fdate >> 9) & 0x7f) + 80;
	month = (((*file_list).ff_fdate >> 5) & 0x0f);
	day   =  ((*file_list).ff_fdate & 0x1f);

	printf(" %02u/%02u/%02u  ", month, day, year);
	return(0);
}
/*-----------------------------------------------------------------------*/
/*ADD_SIZES() - This function adds the file sizes and file count         */
/*-----------------------------------------------------------------------*/
int add_sizes(struct ffblk *file_list)
{
	++count;
	total_size = total_size + (*file_list).ff_fsize;
	return(0);
}
/*-----------------------------------------------------------------------*/
/*PRINT_ATTRIBS() - This function prints the correct attribute for each  */
/*                  file.  If anyone knows an easier way to do this      */
/*                  please let me know!                                  */
/*-----------------------------------------------------------------------*/
void print_attribs(unsigned attributes)
{
	switch(attributes)
		{
		case 0x00 : printf("[N......]\n");
				  break;
		case 0x01 : printf("[.R.....]\n");
				  break;
		case 0x02 : printf("[..H....]\n");
				  break;
		case 0x03 : printf("[.RH....]\n");
				  break;
		case 0x04 : printf("[...S...]\n");
				  break;
		case 0x07 : printf("[.RHS...]\n");    /* Needed for SYSTEM */
				  break;                    /* Why? */
		case 0x08 : printf("[....V..]\n");
				  break;
		case 0x10 : printf("[.....D.]\n");
				  break;
		case 0x20 : printf("[......A]\n");
				  break;
		case 0x27 : printf("[.RH...A]\n");
				  break;
		case 0x28 : printf("[....V.A]\n");
				  break;
		case 0x30 : printf("[.....DA]\n");
				  break;
		case 0x32 : printf("[...SV.A]\n");
				  break;
		default   : printf("\n");
				  break;
		}
}
/*-----------------------------------------------------------------------*/
/*PARSE_FILENAMES() - This function parses the filenames into two parts  */
/*                    so I could justify the filenames for easy reading. */
/*-----------------------------------------------------------------------*/
int parse_filenames(struct ffblk *file_list)
{
	auto char temp_name[14] = "";
	auto char first_name[9] = "";
	auto char last_name[4]  = "";
	auto char *result, length, *length1;
	auto int location  = 0;
	auto int location1 = 0;

	strcpy(temp_name, (*file_list).ff_name);

	location = strcspn(temp_name, ".");

	location1 = location;

     strncat(first_name, temp_name, location);

	while(location1 < 8)
	{
		strcat(first_name, " ");
		++location1;
	}

	printf("%s", first_name);

	result = strrchr(temp_name, '.');

	length = strlen(result);

	if(length == 0)
	{
		strcpy(last_name, ".");
	}
	strncat(last_name, result, length);

	while(length < 4)
	{
		strcat(last_name, " ");
		length++;
	}

	printf("%s", last_name);
	return(0);
}
/*-----------------------------------------------------------------------*/
/*GET_DISK_SPACE() - This function gets disk space off the default disk  */
/*                   The next version will get disk space for a specified*/
/*                   disk(s).                                            */
/*-----------------------------------------------------------------------*/
void get_disk_space(void)
{
	auto unsigned long total_space, free_space, bytes_per_cluster;

	getdfree(0, &dfinfo);

	bytes_per_cluster = dfinfo.df_sclus * dfinfo.df_bsec;

	total_space = dfinfo.df_total * bytes_per_cluster;
	free_space  = dfinfo.df_avail * bytes_per_cluster;

	printf("%10ld bytes free\n%10ld bytes of total.\n", free_space,
											  total_space);
	exit(0);
}
/*-----------------------------------------------------------------------*/
/*ERROR_EXIT() - This function is called if the user enters the wrong    */
/*               option.  Also is used if the user enters the /? option  */
/*-----------------------------------------------------------------------*/
int error_exit(void)
{
	wopen(1,5,24,70,1,3,23);
	wtitle("[HELP SCREEN]",2,9);
	wshadow(8);
	wcenters(1,LCYAN|_BLUE,"XDIR Version 1.10");
	wcenters(2,LCYAN|_BLUE,"Written by Mike Couture");
	wcenters(3,LCYAN|_BLUE,"Copyright (c) 1990  All Rights Reserved");
	wgotoxy(5,0);
	wputs("    XDIR is distributed under the 'Freeware' concept.  This  \n"
		 "    program was first developed in the classroom.  I have    \n"
		 "    since added numerous options.  I utilitized the CXL v5.2 \n"
		 "    add-in library for the many of the advanced functions.   \n"
		 "                                                             \n"
		 "    Valid options are:                                       \n"
		 "                   /d - Display Sub-Directories              \n"
		 "                   /s - Display System Files                 \n"
		 "                   /f - Show Free Disk Space                 \n"
		 "                   /? - This Help Screen                     \n"
		 "                   /r - Display Read-Only Files              \n"
		 " 	    											   \n"
		 "    Example: XDIR /df C:\\MSDOS\\*.EXE                       \n"
		 "                                                             \n"
		 " 	                <<< Press Any Key >>>                       ");
	waitkey();
	wclose();
	return(0);
}
